;; todo: user-agent parameterization support?

(define (main argv) (apply inhale-url (cdr argv)))

(define (inhale-url url)
  (with-exception-catcher
   (lambda (e) (display-exception e) (newline) (list url))
   (lambda () (delete-duplicates
          `(,@(append-map expand-mrl (url->mrls url)) ,url)))))

(define (url->mrls url #!optional (root #t))
  (define (get-url*/ua-fallback . args)
    (define user-agent (str* "curl/7.24.0 (x86_64-apple-darwin12.0) "
                             "libcurl/7.24.0 OpenSSL/0.9.8x zlib/1.2.5"))
    (receive (req status hdrs body)
        (apply get-url* (append args `(user-agent: ,user-agent)))
      (if (equal? "406" status)
          (apply get-url* args)
          (values req status hdrs body))))
  (define (get-url/ua-fallback . args)
    (receive (_ _ _ body) (apply get-url*/ua-fallback args) body))

  (define (parse-asx data)
    (let1 mrls (rxmatch-map
                (rx/i "<\\s*ref[^=]*=\\s*\"?([^\"\n\r]*)\"?") data identity)
      (if (null? mrls) (parse-generic data) mrls)))
  (define (parse-generic data)
    (rxmatch-map (rx/i "([a-z]+://[^ \t\n\r\"<>]+)") data identity))

  (define (sniff-extension url)
    (or (and ((rx/i "\\.(asx|wmx|wax)[^/]*$") url) parse-asx)
        (and (any (lambda (rx) (rx url)) (list (rx/i "\\.(pls|m3u|smil|wax)[^/]*$")
                                          (rx/i "^file://.*")))
             parse-generic)))
  (define (sniff-http-header url)
    (receive (_ status hdrs _)
        (get-url*/ua-fallback url headers-only?: #t)
      (let1 content-any (lambda xs (any (lambda (rx)
                                     ((rx/i rx)
                                      (or (alist-get "content-type" hdrs) "")))
                                   xs))
        (and status
             (not (eq? (string-ref status 0) #\4))
             (cond
              ((content-any "^application/xhtml" "x-ms") parse-asx)
              ((content-any "application/xml" "scpls" "mpegurl") parse-generic)
              ((and (content-any "octet-stream")
                    (alist-get "content-disposition" hdrs)) => sniff-extension)
              (else #f))))))

  (let1 data (delay (or (get-url/ua-fallback url) ""))
    (define (parse url sniffers)
      (and (not (null? sniffers))
           (aif parser ((car sniffers) url)
             (let1 urls (parser (force data))
               (if (null? urls) (parse url (cdr sniffers)) urls))
             (parse url (cdr sniffers)))))

    (let1 urls (parse url (list sniff-extension
                                (if root sniff-http-header (lambda _ #f))))
      (or (and urls (append-map (cut url->mrls <> #f) urls))
          (list url)))))

(define (expand-mrl mrl)
  (rxmatch-let ((rx/i "([^:]+)://([^:/]+:[^@/]+@)?([^:/]+)(:[0-9]+)?(.*)") mrl)
      (#f scheme creds host port rest)
    (let ((scheme-is? (cut string-prefix? <> scheme))
          (maybe (lambda (x) (or x ""))))
      (cond
       ((scheme-is? "mms")
        (map (lambda (x) (str* x "://" (maybe creds) host (maybe port) rest))
             '("mmsh" "mmst" "rtsp")))
       ((scheme-is? "rtmp")
        (let* ((variants ((lambda (e) (list (str* "rtmp" e) (str* "rtmpt" e)))
                          (if (string-contains scheme "e") "e" "")))
               (guess-port (lambda (x) (if ((rx/i "te?$") x) ":80" ":1935")))
               (port (if (string-null? port) (guess-port scheme) port)))
          (append
           (append-map
            (cut map <> variants)
            (list
             (cut str* <> "://" (maybe creds) host port rest)
             (cut str* <> "://" (maybe creds) host rest)
             (lambda (x) (str* x "://" (maybe creds) host (guess-port x) rest)))))))
       (else (list mrl))))))
