

;; todo: report common errors (e.g. "track not recognized")

(define (lfm:make-session key secret user pass)
  (let1 sess (make-url-session)
    (sess 'prop-set! 'api-key key)
    (sess 'prop-set! 'api-secret secret)
    (sess 'prop-set! 'user user)
    (sess 'prop-set! 'auth-token (md5hex (str* user (md5hex pass))))
    (lfm:send-req sess "auth.getMobileSession")
    (values (and (lfm:good? sess)
                 (sess 'prop-set! 'session-key
                       (string-between (sess 'body) "<key>" "</key>"))
                 sess)
            (lfm:fail-reason sess))))

(define (lfm:fail-reason x)
  (let1 ecode (string-between (x 'body) "<error code=\"" "\"")
    (and ecode (case (string->number ecode)
                 ((4) "login")
                 ((8 9 11 16) "try-again")
                 (else "unknown")))))

(define (lfm:good? x)
  (and x (string-contains (x 'body) "lfm status=\"ok\"")))

(define (lfm:nowplaying session artist title)
  (lfm:send-req session
                "track.updateNowPlaying"
                `(("artist" . ,artist) ("track" . ,title))))

(define (lfm:scrobble session artist title song-start-time)
  ;; nb: last.fm won't accept timestamps that look like "123.0" (they
  ;;     demand "123"), so we have to force the timestamp into an
  ;;     exact number at the string-level, since timestamps cannot be
  ;;     represented exactly (as fixnums) on platforms with 32-bit
  ;;     longs
  (define (inexact->exact/string x) (((rx "([0-9]+)\\.?") (str* x)) 1))
  (lfm:send-req session
                "track.scrobble"
                `(("timestamp[0]" . ,(inexact->exact/string song-start-time))
                  ("artist[0]" . ,artist)
                  ("track[0]" . ,title))))

(define (lfm:track-love session artist title)
  (lfm:send-req
   session "track.love" `(("artist" . ,artist) ("track" . ,title))))

(define (lfm:send-req sess method #!optional (extra-fields '()))
  (define (sign alist)
    (cons
     `("api_sig" . ,(md5hex
                     (string-append
                      (apply string-append
                             (map (lambda (x) (str* (car x) (cdr x)))
                                  (sort alist
                                        (lambda (a b) (string<? (car a) (car b))))))
                      (sess 'prop-get 'api-secret))))
     alist))

  (sess 'get "http://ws.audioscrobbler.com/2.0/"
        post-data: (sign
                    `(,@(aif sk (sess 'prop-get 'session-key)
                          `(("sk" . ,sk))
                          `(("authToken" . ,(sess 'prop-get 'auth-token))
                            ("username" . ,(sess 'prop-get 'user))))
                      ,@extra-fields
                      ("api_key" . ,(sess 'prop-get 'api-key))
                      ("method" . ,method))))
  sess)
