;; common stuff
;;

(define-syntax push!
  (syntax-rules ()
    ((_ val lis)
     (set! lis (cons val lis)))))

(define-syntax bound?
  (syntax-rules ()
    ((_ id)
     (with-exception-catcher
      (lambda (e) #f)
      (lambda () id #t)))))

(define-syntax awhen
  (syntax-rules ()
    ((_ id expr body ...)
     (let ((id expr)) (when id body ...)))))

(define-syntax aif
  (syntax-rules ()
    ((_ id expr cons alt) (let ((id expr)) (if id cons alt)))))

(define-syntax let1
  (syntax-rules ()
    ((_ var val body ...)
     (let ((var val)) body ...))))

(define-syntax alist-let
  (syntax-rules ()
    ((_ alist ((a b) ...) body ...)
     (let ((alist_ alist))
       (let ((a (alist-get* 'b alist_)) ...)
         body ...)))

    ((_ alist ((a p b) ...) body ...)
     (let ((alist_ alist))
       (let ((a (aif result (alist-get* 'b alist_) (p result) #f)) ...)
         body ...)))))

(define-syntax multi-lambda
  (syntax-rules ()
    ((_ ((name . formals) body ...) ...)
     (lambda args (apply (case (car args) ((name) (lambda formals body ...)) ...)
                    (cdr args))))))

(define-syntax let/ec
  (syntax-rules ()
    ((_ k body ...)
     (call/cc
      (lambda (k) body ...)))))

;; regex stuff, adapted from Gauche
;;

(define-syntax rxmatch-bind*
  (syntax-rules ()
    [(rxmatch-bind* ?n ?match () ?form ...)
     (begin ?form ...)]
    [(rxmatch-bind* ?n ?match (#f ?vars ...) ?form ...)
     (rxmatch-bind* (+ ?n 1) ?match (?vars ...) ?form ...)]
    [(rxmatch-bind* ?n ?match (?var ?vars ...) ?form ...)
     (let ((?var (?match ?n)))
       (rxmatch-bind* (+ ?n 1) ?match (?vars ...) ?form ...))]))

(define-syntax rxmatch-let
  (syntax-rules ()
    [(rxmatch-let ?expr (?var ...) ?form ...)
     (cond [?expr => (lambda (match) (rxmatch-bind* 0 match (?var ...) ?form ...))]
           [else (error "rxmatch-let: match failed:" '?expr)])]))

(define-syntax rxmatch-if
  (syntax-rules ()
    [(rxmatch-if ?expr (?var ...) ?then ?else)
     (cond [?expr => (lambda (match) (rxmatch-bind* 0 match (?var ...) ?then))]
           [else ?else])]))

(define-syntax rxmatch-cond
  (syntax-rules (test else =>)
    [(rxmatch-cond) #f]
    [(rxmatch-cond (else ?form ...))
     (begin ?form ...)]
    [(rxmatch-cond (test ?expr => ?obj) ?clause ...)
     (cond (?expr => ?obj) (else (rxmatch-cond ?clause ...)))]
    [(rxmatch-cond (test ?expr ?form ...) ?clause ...)
     (if ?expr (begin ?form ...) (rxmatch-cond ?clause ...))]
    [(rxmatch-cond (?matchexp ?bind ?form ...) ?clause ...)
     (rxmatch-if ?matchexp ?bind
       (begin ?form ...)
       (rxmatch-cond ?clause ...))]))

(define-syntax rxmatch-case
  (syntax-rules (test else =>)
    [(rxmatch-case #t ?temp ?strp) #f]
    [(rxmatch-case #t ?temp ?strp (else => ?proc))
     (?proc ?temp)]
    [(rxmatch-case #t ?temp ?strp (else ?form ...))
     (begin ?form ...)]
    [(rxmatch-case #t ?temp ?strp (test ?proc => ?obj) ?clause ...)
     (cond [(?proc ?temp) => ?obj]
           [else (rxmatch-case #t ?temp ?strp ?clause ...)])]
    [(rxmatch-case #t ?temp ?strp (test ?proc ?form ...) ?clause ...)
     (if (?proc ?temp)
       (begin ?form ...)
       (rxmatch-case #t ?temp ?strp ?clause ...))]
    [(rxmatch-case #t ?temp ?strp (?re ?bind ?form ...) ?clause ...)
     (rxmatch-if (and ?strp ((rx ?re) ?temp))
         ?bind
       (begin ?form ...)
       (rxmatch-case #t ?temp ?strp ?clause ...))]
    [(rxmatch-case #t ?temp ?strip ?clause ...)
     (syntax-error "malformed rxmatch-case")]
    ;; main entry
    [(rxmatch-case ?str ?clause ...)
     (let* ([temp ?str]
            [strp (string? temp)])
       (rxmatch-case #t temp strp ?clause ...))]))
