(define (metaf . args)
  ;; see "match" below for usage

  (define (string-titlecase/whatson x)
    (define (word-titlecase word)
      (rxmatch-case word
        ("^(.*?)([/\\-&\\|\\.])(.*)" (#f a sep b)
         (string-append (word-titlecase a) sep (word-titlecase b)))
        ("^([(['\"])(.*)" (#f a b) (string-append a (word-titlecase b)))
        ("^([AF])(\\.?M\\.?)" (#f a r) (string-append a r))
        ("^(.)'(.)(.+)$" (#f a b r)
         (string-append (string-upcase a) "'" (string-upcase b)
                        (string-downcase r)))
        ("^[Mm][Cc](.)(.*)" (#f a r)
         (string-append "Mc" (string-upcase a) (string-downcase r)))
        ("^[IVX]+$" (#f) (string-upcase word)) ;; roman numerals
        ("^(.)(.*)" (#f a r)
         (string-append (string-upcase a) (string-downcase r)))
        (else word)))
    (string-join (map word-titlecase (string-split x (rx "\\s"))) " "))

  (match args
    ;; (metaf '(a ...)) => (metaf a ...)
    (((? list? args)) (apply metaf args))

    ;; (metaf regex string #!key (reverse #f))
    (((? regexp? rx) (? string? data))
     (metaf rx data reverse: #f))
    (((? regexp? rx) (? string? data) reverse: rev)
     (apply metaf ((if rev reverse identity) (rxmatch->list (rx data)))))

    ;; (metaf string1 ...)
    ((((? string? fields) ...)
      (? (cut every pair? <>) alist)
      (? procedure? cleaner))
     (apply metaf (map (lambda (x) (cleaner (alist-get x alist))) fields)))

    ;; (metaf '(string1 ...) alist)
    ((((? string? fields) ...) (? (cut every pair? <>) alist))
     (metaf fields alist identity))

    ;; (metaf 'xtags '(string1 ...) data #!key (reverse #f))
    (('xtags ((? symbol? tags) ...) data)
     (metaf 'xtags tags data reverse: #f))
    (('xtags ((? symbol? tags) ...) data reverse: rev)
     (apply metaf (list (rx/xtags tags) data reverse: rev)))

    (else
     (let* ((needs-unspaced-dash-separation
             (and (null? (cdr args))
                  (let ((x (car args)))
                    (and ((rx "-") x) (not ((rx "( +- +)") x))))))
            (clean (compose
                    string-titlecase/whatson
                    string-trim-both
                    (cut regexp-replace-all*
                         <>
                         `(("-+" . "-")
                           ,@(if needs-unspaced-dash-separation
                                 '(("-" . " - "))
                                 '())
                           (" +- +" . " - ")
                           ("[ _+]+" . " ")
                           ("[\\\\*]" . "")))
                    url-unquote
                    html-unescape)))
       (string-join
        (map clean (filter (lambda (x) (and x (not ((rx "^\\s*$") x)))) args))
        " - ")))))

(define (main argv) (apply metaf (cdr argv)))
