(define (store-lookup iso-country-code terms)
  (define (amazon-lookup terms #!optional (cc ""))
    (define result
      (alist-get*
       '(result track)
       (cdar (xml->sexp (get-url
                        (str* "http://www.amazon.com/gp/dmusic/aws/search.html"
                              (url-quote* `((clientid . "radium")
                                            (field-keywords . ,terms)
                                            (type . TRACK)
                                            (ie . UTF8)))))))))
    (define (field x)
      (define required-url?
        (cute string= (str* "https://www.amazon.com/dmusic/" x) <>))
      (let/ec k
        (find (lambda (y) (match y (`(meta (@ (rel ,(? required-url?))) ,v) (k v))
                              (_ #f)))
              result)))
    (and result `((album . ,(car (alist-get 'album result)))
                  (buyURL . ,(str* "http://www.amazon.com/dp/product/"
                                   (field "ASIN")))
                  (artworkURL . ,(field "imageLarge"))
                  (price . ,(field "price")))))

  (define (itunes-lookup terms #!optional (cc ""))
    (define (clean x)
      (let1 collection-only? (negative? (alist-get "trackPrice" x))
        `(,@(if collection-only?
                `(("trackPrice" . ,(alist-get "collectionPrice" x)))
                '())
          ("collectionOnly" . ,collection-only?)
          ,@x)))
    (define (fetch terms)
      (json-decode
       (string-trim-both
        (get-url
         (str* "http://ax.itunes.apple.com/WebObjects/"
               "MZStoreServices.woa/wa/wsSearch?limit=1&media=music&term="
               (url-quote terms)
               "&country="
               cc)))))
    (let1 info (fetch (regexp-replace-all* terms '(("['\\-.\"]+" . ""))))
      (and (positive? (alist-get "resultCount" info))
           (let1 results (clean (car (alist-get "results" info)))
             (map (lambda (x) (cons (car x) (alist-get (cdr x) results)))
                  '((price . "trackPrice") (albumOnly . "collectionOnly")
                    (buyURL . "trackViewUrl") (previewURL . "previewUrl")
                    (artworkURL . "artworkUrl100") (album . "collectionName")
                    (currency . "currency")))))))

  (define (7digital-lookup terms #!optional (cc ""))
    (let ((result
           (cdadr
            (xml->sexp (get-url
                       (str* "http://api.7digital.com/1.2/track/search?"
                             "oauth_consumer_key=7dz5bqcfhjjd&imageSize=500&"
                             "pageSize=1&country=" cc "&q="
                             (url-quote terms)))))))
      (and (equal? "ok" (car (alist-get* '(@ status) result)))
           (map (lambda (x) (cons (car x)
                             (car (alist-get*
                                   `(searchResults searchResult track ,@(cdr x))
                                   result))))
                '((price . (price value))
                  (currency . (price currency @ code))
                  (buyURL . (url))
                  (artworkURL . (release image))
                  (album . (release title)))))))

  ;; nb: 7digital is disabled because their search engine returns too
  ;;     many false-positives, and it's not clear how to use its
  ;;     "score" metric to reject false-positives. sometimes what
  ;;     we're looking for has a low score, sometimes we get false
  ;;     positives with a high score. need to investigate this
  ;;     further.
  (filter cdr `(;; (7digital . ,(or (7digital-lookup terms iso-country-code)
                ;;                  (7digital-lookup terms)))
                (amazon . ,(amazon-lookup terms))
                (itunes . ,(or (itunes-lookup terms iso-country-code)
                               (itunes-lookup terms))))))

(define (main argv) (apply store-lookup (cdr argv)))
