(define (sxm:verify user pass ctype)
  (define (->xml x)
    ;; nb: this converter is specific to our usage here; shouldn't be
    ;;     used in the general case
    (cond
     ((null? x) '())
     ((and (list? x) (every pair? x)) (apply str* (map ->xml x)))
     ((pair? x) (let ((tag (car x)))
                     (str* "<" tag ">" (->xml (cdr x)) "</" tag ">")))
     (else x)))
  (define (api-url x)
    (str* "https://production-authentication.siriusxm.com"
          "/authentication/en-us/xml/user/login/v3/" x))
  (define (auth-req . data)
    (->xml `(AuthenticationRequest (userName . ,user)
                                  (consumerType . ,ctype)
                                  (subscriberType . SIRIUS_SUBSCRIBER)
                                  (currency . 840)
                                  ,@data)))
  (define (compute-response method challenge iters salt)
    (let ((pkey (crypto:pbkdf2/sha256
                 (digest:hex-string->cbytes
                  (case method
                    ((0) (md5hex pass))
                    ((1) (md5hex (string-upcase pass)))
                    (else "D0DB1CA3B300831A301AF9144FC6986A")))
                 (digest:hex-string->cbytes salt)
                 iters
                 16))
          (hwid "00000000")
          (emac "0000CAFEBABE"))
      (values hwid
              emac
              (digest:hex-encode pkey)
              (digest:hex-encode
               (crypto:aes-encrypt
                pkey
                (digest:hex-string->cbytes (make-string 32 #\0))
                (digest:hex-string->cbytes
                 (str* challenge (string-take
                                  (digest:hex-encode
                                   (digest:sha256
                                    (digest:hex-string->cbytes
                                     (str* hwid emac challenge))))
                                  32))))))))
  (receive (hwid emac pkey response)
      (rxmatch-let
          ((rx/xtags
            '(authenticationchallenge iterationscount passwordhashtype salt))
           (get-url (api-url "initiate") post-data: (auth-req)))
          (_ challenge iters method salt)
        (compute-response
         (string->number method) challenge (string->number iters) salt))
    (rxmatch-let
        ((rx/xtags '(sessionid channellineupid))
         (get-url (api-url "complete")
                  post-data: (auth-req `(playerIdentification
                                         (hardwareIdentification . ,hwid)
                                         (ethernetMac . ,emac))
                                       `(authenticationData . ,response))))
        (_ session lineup)
      (values session lineup pkey))))

(define (sxm:get-channels user pass ctype)
  (receive (_ lineup _) (sxm:verify user pass ctype)
    (let ((prefix "SiriusXM"))
      (rxmatch-map
       (rx/xtags '(contentid displayname name siriuschannelno xmchannelno))
       (get-url (str*
                 "https://www.siriusxm.com/userservices/cl/en-us/xml/lineup/"
                 lineup "/client/" (string-trim-both ctype #\2)))
       (lambda (cid desc name num xm-num)
         (map html-unescape
              (list
               cid
               num
               (if (string-prefix? prefix name) name (str* prefix " " name))
               (regexp-replace (rx "\\s*-XL\\s*") desc "")
               xm-num)))))))

(define (sxm:media-url cid user pass ctype)
  (receive (session _ pkey) (sxm:verify user pass ctype)
    (str* "https://production-token.siriusxm.com:443/token/en-us"
          "/json/v3/streaming/" ctype "/" cid
          (url-quote* `((sessionId . ,session)))
          "///pkey=" pkey)))
